from fast_trade import take_action, determine_action
from collections import namedtuple
import pandas as pd


def test_take_action_greater_than():
    mock_strategy = [["close", ">", "short"]]
    MockRow = namedtuple("MockRow", "date close open high low volume short")
    mock_row = MockRow(
        date=1523937963,
        close=0.0212,
        open=0.01,
        high=0.025,
        low=0.01,
        volume=36898,
        short=0.0112,
    )

    res = take_action(mock_row, mock_strategy)
    assert res is True


def test_take_action_less_than():
    mock_strategy = [["close", "<", "short"]]
    MockRow = namedtuple("MockRow", "date close open high low volume short")
    mock_row = MockRow(
        date=1523937963,
        close=0.0212,
        open=0.01,
        high=0.025,
        low=0.01,
        volume=36898,
        short=0.0112,
    )

    res = take_action(mock_row, mock_strategy)
    assert res is False


def test_take_action_equal():
    mock_strategy = [["close", "=", 0.0212]]
    MockRow = namedtuple("MockRow", "date close open high low volume short")
    mock_row = MockRow(
        date=1523937963,
        close=0.0212,
        open=0.01,
        high=0.025,
        low=0.01,
        volume=36898,
        short=0.0112,
    )

    res = take_action(mock_row, mock_strategy)
    assert res is True



import pandas as pd
from datetime import timedelta


def analyze_df(df: pd.DataFrame, strategy: dict):
    """Analyzes the dataframe and runs sort of a market simulation, entering and exiting positions

    Parameters
    ----------
        df, dataframe from process_dataframe after the actions have been added
        strategy: dict, contains instructions on when to enter/exit trades

    Returns
    -------
        df, returns a dataframe with the new rows processed
    """
    in_trade = False
    last_base = float(strategy["base_balance"])
    commission = float(strategy["commission"])
    last_aux = 0.0
    new_total_value = last_base

    aux_list = []
    base_list = []
    total_value_list = []
    in_trade_list = []
    fee_list = []

    for row in df.itertuples():
        close = row.close
        curr_action = row.action
        fee = 0

        if curr_action == "e" and not in_trade:
            # this means we should enter the trade
            last_aux = convert_base_to_aux(last_base, close)
            fee = calculate_fee(last_aux, commission)

            last_aux = last_aux - fee
            new_total_value = convert_aux_to_base(last_aux, close)

            # should be extremely close to 0
            last_base = round(last_base - new_total_value, 8)
            in_trade = True

        if curr_action == "x" and in_trade:
            last_base = convert_aux_to_base(last_aux, close)
            fee = calculate_fee(last_base, commission)
            last_base = last_base - fee
            last_aux = convert_base_to_aux(last_base, close)
            new_total_value = last_base

            in_trade = False

        aux_list.append(last_aux)
        base_list.append(last_base)
        total_value_list.append(new_total_value)
        in_trade_list.append(in_trade)
        fee_list.append(fee)

    if strategy.get("exit_on_end") and in_trade:
        last_base = convert_aux_to_base(last_aux, close)
        last_aux = convert_base_to_aux(last_base, close)
        new_date = df.index[-1] + timedelta(minutes=1)

        df = df.append(pd.DataFrame(index=[new_date]))

        aux_list.append(last_aux)
        base_list.append(last_base)
        total_value_list.append(new_total_value)
        in_trade_list.append(in_trade)
        fee_list.append(False)

    df["aux"] = aux_list
    df["base"] = base_list
    df["total_value"] = total_value_list
    df["in_trade"] = in_trade_list
    df["fee"] = fee_list

    return df


def convert_base_to_aux(last_base: float, close: float):
    """converts the base coin to the aux coin
    Parameters
    ----------
        last_base, the last amount maintained by the strat
        close, the closing price of the coin

    Returns
    -------
        float, amount of the last base divided by the closing price
    """
    if last_base:
        return round(last_base / close, 8)
    return 0.0


def convert_aux_to_base(last_aux: float, close: float):
    """converts the aux coin to the base coin
    Parameters
    ----------
        last_base, the last amount maintained by the strat
        close, the closing price of the coin
    Returns
    -------
        float, amount of the last aux divided by the closing price
    """
    if last_aux:
        return round(last_aux * close, 8)
    return 0.0


def calculate_fee(price: float, commission: float):
    """calculates the trading fees from the exchange
    Parameters
    ----------
        price, amount of the coin after the transaction
        commission, percentage of the transaction
    """
    if commission:
        return round((price / 100) * commission, 8)

    return 0.0



import datetime
import pandas as pd

from .build_data_frame import build_data_frame
from .run_analysis import analyze_df
from .build_summary import build_summary


def run_backtest(strategy: dict, ohlcv_path: str = "", df: pd.DataFrame = None):
    """
    Parameters
        strategy: dict, required, object containing the logic to test
        ohlcv_path: string or list, required, where to find the csv file of the ohlcv data
        df: pandas dataframe indexed by date
    Returns
        dict
            summary dict, summary of the performace of backtest
            df dataframe, object used in the backtest
            trade_log, dataframe of all the rows where transactions happened
    """

    start = datetime.datetime.utcnow()

    if ohlcv_path:
        df = build_data_frame(strategy, ohlcv_path)
    flagged_enter, flagged_exit, strategy = get_flagged_logiz(strategy)

    strategy["base_balance"] = strategy.get("base_balance", 1000)
    strategy["exit_on_end"] = strategy.get("exit_on_end", True)
    strategy["commission"] = strategy.get("commission", 0)

    df = process_dataframe(df, strategy)

    if flagged_enter or flagged_exit:
        strategy["enter"].extend(flagged_enter)
        strategy["exit"].extend(flagged_exit)

        df = process_dataframe(df, strategy)

    summary, trade_log = build_summary(df, start, strategy)

    return {"summary": summary, "df": df, "trade_df": trade_log, "strategy": strategy}


def get_flagged_logiz(strategy: dict):
    """removes logiz that need to be processed after the initial run

    Parameters
    ----------
        strategy, dictionary of instructions

    Returns
    -------
     tuple, flagged_enter, flagged_exit list of what to reprocess after the first process
        strategy, dict, modified strategy object
    """

    to_flag = ["aux_perc_change"]
    flagged_exit_idx = []
    flagged_enter_idx = []

    flagged_enter = []
    flagged_exit = []

    for idx, ex in enumerate(strategy["exit"]):
        if ex[0] in to_flag or ex[2] in to_flag:
            flagged_exit_idx.append(idx)

    for e_ex in flagged_exit_idx:
        flagged_exit.append(strategy["exit"].pop(e_ex))

    for idx, en in enumerate(strategy["enter"]):
        if en[0] in to_flag or en[2] in to_flag:
            flagged_enter_idx.append(idx)

    for e_en in flagged_enter:
        flagged_enter.append(strategy["enter"].pop(e_en))

    return flagged_enter, flagged_exit, strategy


def process_dataframe(df: pd.DataFrame, strategy: dict):
    """Processes the frame and adds the resultent rows
    Parameters
    ----------
        df, dataframe with all the calculated indicators
        strategy, strategy object

    Returns
    -------
        df, dataframe with with all the actions and strategy processed
    """
    df["action"] = [determine_action(frame, strategy) for frame in df.itertuples()]

    df = analyze_df(df, strategy)

    df["aux_perc_change"] = df["total_value"].pct_change() * 100
    df["aux_change"] = df["total_value"].diff()

    return df


def determine_action(frame: pd.DataFrame, strategy: dict):
    """processes the actions with the applied logic
    Parameters
    ----------
        frame: current row of the dataframe
        strategy: object with the logic of how to trade
        df_col_map: dictionary with the column name and index
        of the dataframe

    Returns
    -------
        string, "e" (enter), "x" (exit), "h" (hold) of what
        the strategy would do
    """

    if take_action(frame, strategy["enter"]):
        return "e"

    if take_action(frame, strategy["exit"]):
        return "x"

    return "h"


def take_action(row, strategy):
    """determines whether to take action based on the logic in the strategy
    Parameters
    ----------
        row: data row to operate on
        strategy: dictionary of logic and how to impliment it

    Returns
    -------
        boolean, True if row meets the criteria of given strategy,
        False if otherwise
    """
    results = []
    row = row._asdict()
    for each in strategy:
        val0 = (
            each[0]
            if isinstance(each[0], int) or isinstance(each[0], float)
            else row[each[0]]
        )
        val1 = (
            each[2]
            if isinstance(each[2], int) or isinstance(each[2], float)
            else row[each[2]]
        )

        if isinstance(val0, pd.Series):
            val0 = row[each[0]].values[0]

        if isinstance(val1, pd.Series):
            val1 = row[each[2]].values[0]

        if each[1] == ">":
            results.append(bool(val0 > val1))
        if each[1] == "<":
            results.append(bool(val0 < val1))
        if each[1] == "=":
            results.append(bool(val0 == val1))
        if each[1] == "!=":
            results.append(bool(val0 != val1))

    return all(results)



import pytest
import pandas as pd
import random
from fast_trade import convert_base_to_aux, convert_aux_to_base, analyze_df


def test_convert_base_to_aux_1():
    mock_close = 10
    mock_last_base = 100
    res = convert_base_to_aux(mock_last_base, mock_close)

    assert res == 10


def test_convert_base_to_aux_2():
    mock_close = 0.025
    mock_last_base = 60

    res = convert_base_to_aux(mock_last_base, mock_close)
    assert res == 2400.0


def test_convert_base_to_aux_3():
    mock_close = 0.3123
    mock_last_base = 212.2333

    res = convert_base_to_aux(mock_last_base, mock_close)
    assert res == 679.58149215


def test_convert_base_to_aux_4():
    mock_close = 0.3123
    mock_last_base = 0

    res = convert_base_to_aux(mock_last_base, mock_close)
    assert res == 0


def test_convert_base_to_aux_0s():
    mock_close = 0
    mock_last_base = 0

    res = convert_base_to_aux(mock_last_base, mock_close)
    assert res == 0


def test_convert_aux_to_base_1():
    mock_close = 0.99992
    mock_last_aux = 133.22

    aux = convert_aux_to_base(mock_last_aux, mock_close)

    assert aux == 133.2093424


def test_convert_aux_to_base_2():
    mock_close = 0.99992
    mock_base_balance = 0

    aux = convert_aux_to_base(mock_close, mock_base_balance)

    assert aux == 0.0


def test_analyze_df_1():
    mock_df = pd.read_csv("./test/ohlcv_data.csv.txt", parse_dates=True).set_index(
        "date"
    )
    mock_df.index = pd.to_datetime(mock_df.index, unit="s")
    mock_strategy = {"base_balance": 1000, "exit_on_end": True, "commission": 0.00}

    mock_df["action"] = ["e", "h", "x", "x", "x", "e", "x", "h", "h"]

    df = analyze_df(mock_df, mock_strategy)

    print(df.action)
    assert list(df.aux.values) == [
        100000.0,
        100000.0,
        100000.0,
        100000.0,
        100000.0,
        103237.41007194,
        103237.41007172,
        103237.41007172,
        103237.41007172,
    ]
    assert list(df.base.values) == [
        0,
        0,
        2296.0,
        2296.0,
        2296.0,
        0,
        2274.32014388,
        2274.32014388,
        2274.32014388,
    ]
    assert list(df.total_value.values) == [
        1000.0,
        1000.0,
        2296.0,
        2296.0,
        2296.0,
        2296.0,
        2274.32014388,
        2274.32014388,
        2274.32014388,
    ]


def test_analyze_df_2():
    mock_df = pd.read_csv("./test/ohlcv_data.csv.txt", parse_dates=True).set_index(
        "date"
    )
    mock_df.index = pd.to_datetime(mock_df.index, unit="s")

    mock_strategy = {"base_balance": 1000, "exit_on_end": True, "commission": 0.00}
    mock_df["action"] = ["e", "h", "h", "x", "h", "h", "e", "h", "h"]

    df = analyze_df(mock_df, mock_strategy)

    assert list(df.aux.values) == [
        100000.0,
        100000.0,
        100000.0,
        100000.0,
        100000.0,
        100000.0,
        96232.41034952,
        96232.41034952,
        96232.41034952,
        96232.41034949,
    ]
    assert list(df.base.values) == [
        0,
        0,
        0,
        2120.0,
        2120.0,
        2120.0,
        0,
        0,
        0,
        2065.1475261,
    ]
    assert list(df.total_value.values) == [
        1000.0,
        1000.0,
        1000.0,
        2120.0,
        2120.0,
        2120.0,
        2120.0,
        2120.0,
        2120.0,
        2120.0,
    ]



import pandas as pd
from finta import TA
import os

DATE_FORMAT = "%Y-%m-%d %H:%M:%S"


def build_data_frame(strategy: dict, csv_path: str):
    """Creates a Pandas DataFame with the provided strategy. Used when providing a CSV as the datafile

    Parameters
    ----------
    strategy: dict, provides instructions on how to build the dataframe
    csv_path: string, absolute path of where to find the data file

    Returns
    -------
    object, A Pandas DataFrame indexed buy date
    """
    df = load_basic_df_from_csv(csv_path)

    df = prepare_df(df, strategy)

    if df.empty:
        raise Exception("Dataframe is empty. Check the start and end dates")

    return df


def load_basic_df_from_csv(csv_path: str):
    """Loads a dataframe from a csv
    Parameters
    ----------
        csv_path: string, path to the csv so it can be read

    Returns
        df, A basic dataframe with the data from the csv
    """

    if not os.path.isfile(csv_path):
        raise Exception(f"File not found: {csv_path}")
    cols = [0, 1, 2, 3, 4, 5]
    df = pd.read_csv(csv_path, parse_dates=True, usecols=cols)
    df = df.set_index("date")
    df = df.drop_duplicates()
    time_unit = detect_time_unit(df.iloc[0].values[0])
    df.index = pd.to_datetime(df.index, unit=time_unit)

    return df


def prepare_df(df: pd.DataFrame, strategy: dict):
    """Prepares the provided dataframe for a backtest by applying the indicators and splicing based on the given strategy.
        Useful when loading an existing dataframe (ex. from a cache).

    Parameters
    ----------
        df: DataFrame, should have all the open, high, low, close, volume data set as headers and indexed by date
        strategy: dict, provides instructions on how to build the dataframe

    Returns
    ------
        df: DataFrame, with all the indicators as column headers and trimmed to the provided time frames
    """

    indicators = strategy.get("indicators", [])
    df = apply_indicators_to_dataframe(df, indicators)

    chart_period = strategy.get("chart_period", "1Min")

    start_time = strategy.get("start")
    stop_time = strategy.get("stop")
    df = apply_charting_to_df(df, chart_period, start_time, stop_time)

    return df


def add_freq(idx, freq=None):
    """Add a frequency attribute to idx, through inference or directly.

    Returns a copy.  If `freq` is None, it is inferred.
    """

    idx = idx.copy()
    if freq is None:
        if idx.freq is None:
            freq = pd.infer_freq(idx)
        else:
            return idx
    idx.freq = pd.tseries.frequencies.to_offset(freq)
    if idx.freq is None:
        raise AttributeError(
            "no discernible frequency found to `idx`.  Specify"
            " a frequency string with `freq`."
        )
    return idx


def apply_charting_to_df(
    df: pd.DataFrame, chart_period: str, start_time: str, stop_time: str
):
    """Modifies the dataframe based on the chart_period, start dates and end dates
    Parameters
    ----------
        df: dataframe with data loaded
        chart_period: string, describes how often to sample data, default is '1M' (1 minute)
            see https://pandas.pydata.org/pandas-docs/stable/user_guide/timeseries.html#dateoffset-objects
        start_time: datestring in YYYY-MM-DD HH:MM (ex. 2020-08-31 04:00) of when to begin the backtest
        stop_time: datestring of YYYY-MM-DD HH:MM when to stop the backtest
    Returns
        DataFrame, a sorted dataframe ready for consumption by run_backtest
    """

    if not isinstance(df.index, pd.DatetimeIndex):
        time_unit = detect_time_unit(df.iloc[-1].values[0])
        df.index = pd.to_datetime(df.index, unit=time_unit)
    else:
        df.index = pd.to_datetime(df.index)

    df = df.resample(chart_period).first()

    if start_time and stop_time:
        df = df[start_time:stop_time]  # noqa
    elif start_time and not stop_time:
        df = df[start_time:]  # noqa
    elif not start_time and stop_time:
        df = df[:stop_time]

    return df


def apply_indicators_to_dataframe(df: pd.DataFrame, indicators: list):
    """Applies indications from the strategy to the dataframe
    Parameters
    ----------
        df: dataframe loaded with data
        indicators: list of indictors as dictionary objects

        Indicator detail:
        {
            "func": "", string, actual function to be called MUST be in the indicator_map
            "name": "", string, name of the indicator, becomes a column on the dataframe
            "args": [], list arguments to pass the the function
            "df": string, the name of the dataframe column to be used
        }

    Returns
    -------
        df, a modified dataframe with all the indicators calculated as columns
    """
    for ind in indicators:
        func = ind.get("func")
        field_name = ind.get("name")
        if ind.get("args"):
            args = ind.get("args")
            df[field_name] = indicator_map[func](df, *args)
        else:
            df[field_name] = indicator_map[func](df)

    return df


def detect_time_unit(timestamp: int):
    """Detects whether the timestamp is in seconds or milliseconds

    Parameters
        timestamp, usually a np.int64
    Returns
        string
    """
    if len(str(timestamp)) == 10:
        return "s"

    return "ms"


"""
These are all the indicators the can be used in a strategy as a "func".
Any function can be implimented as an indicator.
"""
indicator_map = {
    "ta.sma": TA.SMA,
    "ta.smm": TA.SMM,
    "ta.ssma": TA.SSMA,
    "ta.ema": TA.EMA,
    "ta.dema": TA.DEMA,
    "ta.tema": TA.TEMA,
    "ta.trima": TA.TRIMA,
    "ta.vama": TA.VAMA,
    "ta.er": TA.ER,
    "ta.kama": TA.KAMA,
    "ta.zlema": TA.ZLEMA,
    "ta.wma": TA.WMA,
    "ta.hma": TA.HMA,
    "ta.evwma": TA.EVWMA,
    "ta.vwap": TA.VWAP,
    "ta.smma": TA.SMMA,
    "ta.macd": TA.MACD,
    "ta.ppo": TA.PPO,
    "ta.vw_macd": TA.VW_MACD,
    "ta.ev_macd": TA.EV_MACD,
    "ta.mom": TA.MOM,
    "ta.roc": TA.ROC,
    "ta.rsi": TA.RSI,
    "ta.ift_rsi": TA.IFT_RSI,
    "ta.tr": TA.TR,
    "ta.atr": TA.ATR,
    "ta.sar": TA.SAR,
    "ta.bbands": TA.BBANDS,
    "ta.bbwidth": TA.BBWIDTH,
    "ta.percent_b": TA.PERCENT_B,
    "ta.kc": TA.KC,
    "ta.do": TA.DO,
    "ta.dmi": TA.DMI,
    "ta.adx": TA.ADX,
    "ta.pivot": TA.PIVOT,
    "ta.pivot_fib": TA.PIVOT_FIB,
    "ta.stoch": TA.STOCH,
    "ta.stochd": TA.STOCHD,
    "ta.stochrsi": TA.STOCHRSI,
    "ta.williams": TA.WILLIAMS,
    "ta.uo": TA.UO,
    "ta.ao": TA.AO,
    "ta.mi": TA.MI,
    "ta.vortex": TA.VORTEX,
    "ta.kst": TA.KST,
    "ta.tsi": TA.TSI,
    "ta.tp": TA.TP,
    "ta.adl": TA.ADL,
    "ta.chaikin": TA.CHAIKIN,
    "ta.mfi": TA.MFI,
    "ta.obv": TA.OBV,
    "ta.wobv": TA.WOBV,
    "ta.vzo": TA.VZO,
    "ta.pzo": TA.PZO,
    "ta.efi": TA.EFI,
    "ta.cfi": TA.CFI,
    "ta.ebbp": TA.EBBP,
    "ta.emv": TA.EMV,
    "ta.cci": TA.CCI,
    "ta.copp": TA.COPP,
    "ta.basp": TA.BASP,
    "ta.baspn": TA.BASPN,
    "ta.cmo": TA.CMO,
    "ta.chandelier": TA.CHANDELIER,
    "ta.qstick": TA.QSTICK,
    "ta.tmf": TA.TMF,
    "ta.fish": TA.FISH,
    "ta.ichimoku": TA.ICHIMOKU,
    "ta.apz": TA.APZ,
    "ta.vr": TA.VR,
    "ta.sqzmi": TA.SQZMI,
    "ta.vpt": TA.VPT,
    "ta.fve": TA.FVE,
    "ta.vfi": TA.VFI,
    "ta.msd": TA.MSD,
    # "ta.wto": TA.WTO, needs a helper function
}



import pytest
import pandas as pd
import datetime

from fast_trade.build_data_frame import (
    detect_time_unit,
    load_basic_df_from_csv,
    apply_indicators_to_dataframe,
    apply_charting_to_df,
)


def test_detect_time_unit_s():
    mock_timestring = 1595115901734
    result = detect_time_unit(mock_timestring)

    assert result == "ms"


def test_detect_time_unit_ms():
    mock_timestring = 1595115901
    result = detect_time_unit(mock_timestring)

    assert result == "s"


def test_load_basic_df_from_csv_str_1():
    mock_ohlcv_path = "./test/ohlcv_data.csv.txt"

    result_df = load_basic_df_from_csv(mock_ohlcv_path)
    header = list(result_df.head())
    print(header)
    assert "close" in header
    assert "open" in header
    assert "high" in header
    assert "low" in header
    assert "volume" in header

    assert result_df.index.name == "date"


def test_load_basic_df_from_csv_list_1():
    mock_ohlcv_path = "./test/ohlcv_data.csv.txt"

    result_df = load_basic_df_from_csv(mock_ohlcv_path)

    expected_line = [0.01404, 0.01, 0.025, 0.01, 3117.0]

    assert list(result_df.iloc[1]) == expected_line


def test_load_basic_df_from_csv_str_error_1():
    mock_ohlcv_path = "./test/SomeFakeNews.csv.txt"

    with pytest.raises(Exception, match=r"File not found:*"):
        load_basic_df_from_csv(mock_ohlcv_path)


def test_apply_indicators_to_dataframe_1_ind():
    mock_df = pd.read_csv("./test/ohlcv_data.csv.txt")
    mock_indicators = [
        {"func": "ta.sma", "name": "example_indicator_name", "args": [3]}
    ]

    result_df = apply_indicators_to_dataframe(mock_df, mock_indicators)

    header = list(result_df.head())

    assert "example_indicator_name" in header
    assert "FAKE_indicator_name" not in header


def test_apply_indicators_to_dataframe_no_args():
    mock_df = pd.read_csv("./test/ohlcv_data.csv.txt")
    mock_indicators = [{"func": "ta.rsi", "name": "rsi", "args": []}]

    result_df = apply_indicators_to_dataframe(mock_df, mock_indicators)

    assert "rsi" in list(result_df.columns)


def test_apply_charting_to_df_1():
    mock_df = pd.read_csv("./test/ohlcv_data.csv.txt", index_col="date")
    # mock_df.set_index(["date"], inplace=True)
    mock_df.index = pd.to_datetime(mock_df.index, unit="s")
    print(mock_df)
    mock_chart_period = "2Min"
    mock_start_time = "2018-04-17"
    mock_stop_time = ""

    result_df = apply_charting_to_df(
        mock_df, mock_chart_period, mock_start_time, mock_stop_time
    )

    assert (result_df.iloc[2].name - result_df.iloc[1].name).total_seconds() == 120
    assert (result_df.iloc[4].name - result_df.iloc[1].name).total_seconds() == 360


def test_apply_charting_to_df_2():
    mock_df = pd.read_csv("./test/ohlcv_data.csv.txt")
    mock_df.set_index(["date"], inplace=True)
    mock_df.index = pd.to_datetime(mock_df.index, unit="s")
    mock_chart_period = "1Min"
    mock_start_time = "2018-04-17 04:00:00"
    mock_stop_time = "2018-04-17 04:10:00"

    past_stop_time = datetime.datetime.strptime(
        "2018-04-17 04:11:00", "%Y-%m-%d %H:%M:%S"
    )
    result_df = apply_charting_to_df(
        mock_df, mock_chart_period, mock_start_time, mock_stop_time
    )

    assert result_df.iloc[-1].name < past_stop_time


def test_apply_charting_to_df_3():
    mock_df = pd.read_csv("./test/ohlcv_data.csv.txt")
    mock_df.set_index(["date"], inplace=True)
    mock_df.index = pd.to_datetime(mock_df.index, unit="s")
    mock_chart_period = "1Min"
    mock_start_time = ""
    mock_stop_time = "2018-04-17 04:10:00"

    past_stop_time = "2018-04-17 04:11:00"

    result_df = apply_charting_to_df(
        mock_df, mock_chart_period, mock_start_time, mock_stop_time
    )

    past_stop_time = datetime.datetime.strptime(
        "2018-04-17 04:11:00", "%Y-%m-%d %H:%M:%S"
    )
    result_df = apply_charting_to_df(
        mock_df, mock_chart_period, mock_start_time, mock_stop_time
    )

    assert result_df.index[0] < past_stop_time



import pathlib
from setuptools import setup, find_packages

HERE = pathlib.Path(__file__).parent

README = (HERE / "README.md").read_text()

setup(
    name="fast-trade",
    version="0.1.6.2",
    description="Automate and backtest on ohlcv data quickly",
    long_description=README,
    long_description_content_type="text/markdown",
    url="https://github.com/jrmeier/fast-trade",
    py_modules=["fast_trade"],
    keywords=[
        "backtesting",
        "currency",
        "ta",
        "pandas",
        "finance",
        "numpy",
        "analysis",
        "technical analysis",
    ],
    author="Jed Meier",
    author_email="fast_trade@jedm.dev",
    license="MIT",
    python_requires=">=3",
    entry_points={"console_scripts": ["ft = fast_trade.cli:main"]},
    classifiers=[
        "Development Status :: 3 - Alpha",
        "Intended Audience :: Financial and Insurance Industry",
        "Programming Language :: Python :: 3.8",
        "Operating System :: OS Independent",
        "Natural Language :: English",
        "License :: OSI Approved :: MIT License",
    ],
    packages=find_packages(include=["fast_trade"]),
    include_package_data=True,
    install_requires=[
        "certifi==2020.6.20",
        "cycler==0.10.0",
        "finta==1.0",
        "kiwisolver==1.2.0",
        "matplotlib==3.3.1",
        "numpy==1.19.1",
        "pandas==1.1.1",
        "Pillow==7.2.0",
        "pyparsing==2.4.7",
        "python-dateutil==2.8.1",
        "pytz==2020.1",
        "six==1.15.0",
    ],
)



import datetime
import numpy as np
import math


def create_trade_log(df):
    """Finds all the rows when a trade was entered or exited

    Parameters
    ----------
        df: dataframe, from process_dataframe

    Returns
    -------
        trade_log_df: dataframe, of when transactions took place
    """

    trade_log = df.reset_index()
    trade_log = trade_log.groupby(
        (trade_log["in_trade"] != trade_log["in_trade"].shift()).cumsum()
    ).first()
    if "date" in trade_log.columns:
        trade_log = trade_log.set_index("date")
    else:
        trade_log = trade_log.set_index("index")

    return trade_log


def build_summary(df, perf_start_time, strategy):
    # Not yet implimented
    # Expectancy [%]                           6.92
    # SQN                                      1.77
    # Sharpe Ratio                             0.22
    # Sortino Ratio                            0.54
    # Calmar Ratio                             0.07

    trade_log = create_trade_log(df)

    trade_time_held_series = trade_log.index.to_series().diff()
    mean_trade_time_held = trade_time_held_series.mean()
    max_trade_time_held = trade_time_held_series.max()
    min_trade_time_held = trade_time_held_series.min()
    median_time_held = trade_time_held_series.median()

    trade_perc_series = trade_log.replace([np.inf, -np.inf], np.nan)

    max_trade_perc = trade_perc_series.max().aux_perc_change
    min_trade_perc = trade_perc_series.min().aux_perc_change
    mean_trade_perc = trade_perc_series.mean().aux_perc_change
    median_trade_perc = trade_perc_series.median().aux_perc_change

    win_trades = trade_log[trade_log.aux_perc_change >= 0]
    loss_trades = trade_log[trade_log.aux_perc_change < 0]

    total_trades = len(trade_log.index)

    try:
        win_perc = (len(win_trades) / total_trades) * 100
    except ZeroDivisionError:
        win_perc = 0
    except TypeError:
        win_perc = 0
    try:
        loss_perc = (len(loss_trades) / total_trades) * 100
    except ZeroDivisionError:
        loss_perc = 0

    if trade_log.iloc[0].total_value and trade_log.iloc[-1].total_value:
        return_perc = 100 - trade_log.iloc[0].total_value / (
            trade_log.iloc[-1].total_value / 100
        )
    else:
        return_perc = 0

    equity_peak = df["total_value"].max()

    equity_final = df.iloc[-1]["total_value"]

    perf_stop_time = datetime.datetime.utcnow()
    start_date = df.index[0]
    end_date = df.index[-1]
    loss_perc = (len(loss_trades) / total_trades) * 100

    first_close = df.iloc[0].close
    last_close = df.iloc[-1].close

    if math.isnan(last_close):
        last_close = df.iloc[-2].close

    buy_and_hold_perc = (1 - (first_close / last_close)) * 100

    summary = {
        "return_perc": round(return_perc, 3),
        "buy_and_hold_perc": round(buy_and_hold_perc, 3),
        "median_trade_len": median_time_held.total_seconds(),
        "mean_trade_len": mean_trade_time_held.total_seconds(),
        "max_trade_held": max_trade_time_held.total_seconds(),
        "min_trade_len": min_trade_time_held.total_seconds(),
        "best_trade_perc": round(max_trade_perc, 3),
        "min_trade_perc": round(min_trade_perc, 3),
        "median_trade_perc": round(median_trade_perc, 3),
        "mean_trade_perc": round(mean_trade_perc, 3),
        "num_trades": total_trades,
        "win_perc": round(win_perc, 3),
        "loss_perc": round(loss_perc, 3),
        "equity_peak": round(float(equity_peak), 3),
        "equity_final": round(float(equity_final), 3),
        "first_tic": start_date.strftime("%Y-%m-%d %H:%M:%S"),
        "last_tic": end_date.strftime("%Y-%m-%d %H:%M:%S"),
        "total_tics": len(df.index),
        "test_duration": (perf_stop_time - perf_start_time).total_seconds(),
    }

    return summary, trade_log
